'use strict';

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

var _sinon = require('sinon');

var _sinon2 = _interopRequireDefault(_sinon);

var _chaiAsPromised = require('chai-as-promised');

var _chaiAsPromised2 = _interopRequireDefault(_chaiAsPromised);

var _immutable = require('immutable');

var _Form = require('./Form');

var _Form2 = _interopRequireDefault(_Form);

var _FormCollection = require('./FormCollection');

var _FormCollection2 = _interopRequireDefault(_FormCollection);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_chai2.default.use(_sinonChai2.default); /* eslint-disable no-unused-expressions */
/* eslint-disable no-new */

_chai2.default.use(_chaiAsPromised2.default);

describe('Form', function () {
  describe('constructor', function () {
    beforeEach(function () {
      _FormCollection2.default.reset();
    });
    it('adds itself to FormCollection on initialization', function () {
      new _Form2.default('test');
      var form = _FormCollection2.default.get('test');
      (0, _chai.expect)(form).to.be.ok;
      (0, _chai.expect)(form.name).to.eql('test');
    });
    it('does not add itself to FormCollection if option is disabled', function () {
      new _Form2.default('test', null, { addToFormCollection: false });
      var form = _FormCollection2.default.get('test');
      (0, _chai.expect)(form).to.eql(undefined);
    });
    it('must have a name', function () {
      (0, _chai.expect)(function () {
        return new _Form2.default();
      }).to.throw(Error);
      (0, _chai.expect)(function () {
        return new _Form2.default('');
      }).to.throw(Error);
      (0, _chai.expect)(function () {
        return new _Form2.default(' ');
      }).to.throw(Error);
    });
    it('can use initial state', function () {
      var initialState = {
        fields: {
          field1: {
            value: 'value1',
            errors: ['error1']
          }
        },
        errors: ['error2']
      };

      var form = new _Form2.default('test', initialState);

      var state = form.getState();
      (0, _chai.expect)(state.getIn(['fields', 'field1', 'value'])).to.eql('value1');
      (0, _chai.expect)(state.getIn(['fields', 'field1', 'errors']).first()).to.eql('error1');
      (0, _chai.expect)(state.get('errors').first()).to.eql('error2');
      // Make sure initialState isn't mutated by Form
      (0, _chai.expect)(initialState).to.eql({
        fields: {
          field1: {
            value: 'value1',
            errors: ['error1']
          }
        },
        errors: ['error2']
      });
    });
  });
  describe('fields', function () {
    it('set field value', function () {
      var form = new _Form2.default('form');
      form.setField('field1', 'value1');
      var field = form.store.getState().getIn(['form', 'fields', 'field1']);
      (0, _chai.expect)(field.get('value')).to.eql('value1');
    });
    it('sets field value, errors', function () {
      var form = new _Form2.default('form');
      form.setField('field1', 'value1', 'error1');
      var field = form.store.getState().getIn(['form', 'fields', 'field1']);
      (0, _chai.expect)(field.get('value')).to.eql('value1');
      (0, _chai.expect)(field.get('errors').first()).to.eql('error1');
    });
    it('clears errors if passed null', function () {
      var form = new _Form2.default('form');
      form.setField('field1', 'value1', 'error1');
      form.setField('field1', null, 'error2');
      var field = form.store.getState().getIn(['form', 'fields', 'field1']);
      (0, _chai.expect)(field.get('errors').size).to.eql(2);
      form.setField('field1', null, null);
      field = form.store.getState().getIn(['form', 'fields', 'field1']);
      (0, _chai.expect)(field.get('errors').size).to.eql(0);
    });
    it('get field', function () {
      var form = new _Form2.default('form');
      form.setField('field1', 'value1', 'error1');
      var field = form.getField('field1');
      (0, _chai.expect)(field.get('value')).to.eql('value1');
    });
    it('remove field', function () {
      var form = new _Form2.default('form');
      form.setField('field1', 'value1');
      form.removeField('field1');
      (0, _chai.expect)(form.getField('field1')).to.eql((0, _immutable.Map)({
        value: '',
        errors: (0, _immutable.Stack)()
      }));
    });
    it('reset field', function () {
      var form = new _Form2.default('form');
      form.setField('field1', 'value1', 'error1');
      form.resetField('field1');
      var field = form.getField('field1');
      (0, _chai.expect)(field.get('value')).to.eql('');
      (0, _chai.expect)(field.get('errors').size).to.eql(0);
    });
    it('returns empty map for field that does not exist', function () {
      var form = new _Form2.default('form');
      var field = form.getField('field1');
      (0, _chai.expect)(field).to.eql((0, _immutable.Map)({
        value: '',
        errors: (0, _immutable.Stack)()
      }));
    });
    it('getFieldValues', function () {
      var form = new _Form2.default('test', {
        fields: {
          field1: {
            value: 'value1'
          },
          field2: {
            value: 'value2'
          },
          field3: {}
        }
      });
      (0, _chai.expect)(form.getFieldValues()).to.eql((0, _immutable.Map)({
        field1: 'value1',
        field2: 'value2',
        field3: ''
      }));
    });
  });
  describe('errors', function () {
    it('add error', function () {
      var form = new _Form2.default('form');
      form.addError('error1');
      (0, _chai.expect)(form.getState().get('errors').first()).to.eql('error1');
    });
    it('clear errors', function () {
      var form = new _Form2.default('form');
      form.setField('field1', null, 'error1');
      form.addError('error2');
      form.clearErrors();
      (0, _chai.expect)(form.getState().get('errors').size).to.eql(0);
    });
    it('has errors - form level', function () {
      var form = new _Form2.default('form');
      form.addError('error1');
      (0, _chai.expect)(form.hasErrors()).to.eql(true);
    });
    it('has errors - field level', function () {
      var form = new _Form2.default('form');
      form.setField('field1', 'value1', 'error1');
      (0, _chai.expect)(form.hasErrors()).to.eql(true);
    });
    it('has errors - form and filed', function () {
      var form = new _Form2.default('form');
      form.setField('field1', 'value1', 'error1');
      var field = form.getField('field1');
      (0, _chai.expect)(field.get('errors').first()).to.eql('error1');
      (0, _chai.expect)(form.hasErrors()).to.eql(true);
    });
    it('getFormErrors', function () {
      var form = new _Form2.default('form');
      form.addError('error1');
      form.addError('error2');
      (0, _chai.expect)(form.getFormErrors()).to.eql((0, _immutable.Stack)(['error2', 'error1']));
    });
  });
  describe('submit', function () {
    it('resolves if form has no errors, calls onSuccess', function (done) {
      var initialState = {
        fields: {
          field1: {
            value: 'value1'
          },
          field2: {
            value: 'value2'
          }
        }
      };

      var form = new _Form2.default('form', initialState);

      form.onSuccess = _sinon2.default.spy();

      var promise = new Promise(function (resolve) {
        resolve('good');
      });

      form.submit(promise).then(function (res) {
        (0, _chai.expect)(res).to.eql('good');
        (0, _chai.expect)(form.onSuccess).to.have.been.called;
        done();
      });
    });
    it('can use promise from setSubmit', function (done) {
      var initialState = {
        fields: {
          field1: {
            value: 'value1'
          },
          field2: {
            value: 'value2'
          }
        }
      };

      var form = new _Form2.default('form', initialState);

      var promise = new Promise(function (resolve) {
        resolve('good');
      });

      form.setSubmit(promise);

      form.submit().then(function (res) {
        (0, _chai.expect)(res).to.eql('good');
        done();
      });
    });
    it('rejects if form has errors', function (done) {
      var initialState = {
        fields: {
          field1: {
            value: 'value1'
          },
          field2: {
            value: 'value2',
            validate: [function () {
              return 'error';
            }]
          }
        }
      };

      var form = new _Form2.default('form', initialState);

      var promise = new Promise(function (resolve) {
        resolve('good');
      });

      form.setSubmit(function () {
        return promise;
      });

      form.submit().catch(function (err) {
        (0, _chai.expect)(err).to.eql('Validation failed');
        (0, _chai.expect)(form.getField('field2').get('errors').first()).to.eql('error');
        done();
      });
    });
    it('rejects and calls on failure if promise fails', function (done) {
      var initialState = {
        fields: {
          field1: {
            value: 'value1'
          },
          field2: {
            value: 'value2'
          }
        }
      };

      var form = new _Form2.default('form', initialState);

      form.onFailure = _sinon2.default.spy();

      var promise = new Promise(function (resolve, reject) {
        reject('bad');
      });

      form.submit(promise).catch(function (err) {
        (0, _chai.expect)(err).to.eql('bad');
        (0, _chai.expect)(form.onFailure).to.have.been.called;
        done();
      });
    });
  });
  describe('filterValidate', function () {
    it('filters validate keys', function () {
      var initialState = {
        fields: {
          field1: {
            value: 'value1',
            validate: function validate() {
              return 'error';
            }
          },
          field2: {
            value: 'value2',
            validate: function validate() {
              return 'error';
            }
          }
        },
        errors: ['error'],
        validate: function validate() {
          return 'error';
        }
      };

      var _filterValidate = (0, _Form.filterValidate)(initialState),
          filteredState = _filterValidate.filteredState,
          fieldValidators = _filterValidate.fieldValidators,
          formValidators = _filterValidate.formValidators;

      (0, _chai.expect)(formValidators).to.be.array;
      (0, _chai.expect)(fieldValidators.field1).to.be.array;
      (0, _chai.expect)(fieldValidators.field2).to.be.array;
      (0, _chai.expect)(filteredState).to.eql({
        fields: {
          field1: {
            value: 'value1'
          },
          field2: {
            value: 'value2'
          }
        },
        errors: ['error']
      });
    });
  });
  describe('validate', function () {
    it('can validate form', function () {
      var initialState = {
        fields: {
          field1: {
            validate: [function () {
              return 'error1';
            }, function () {
              return 'error2';
            }]
          },
          field2: {
            value: 'value2'
          }
        },
        validate: function validate() {
          return 'error3';
        }
      };
      var form = new _Form2.default('test', initialState);
      var isValid = form.validate();
      (0, _chai.expect)(isValid).to.eql(false);
      var state = form.getState();
      (0, _chai.expect)(state.getIn(['fields', 'field1', 'errors']).size).to.eql(2);
      (0, _chai.expect)(state.get('errors').size).to.eql(1);
    });
  });
  describe('addFieldValidator, removeFieldValidator', function () {
    it('can add and remove a form field validator', function () {
      var initialState = {
        fields: {
          field1: {}
        }
      };
      var form = new _Form2.default('test', initialState);
      (0, _chai.expect)(form.validate()).to.eql(true);
      var validator = function validator() {
        return 'error';
      };
      form.addFieldValidator('field1', validator);
      (0, _chai.expect)(form.validate()).to.eql(false);
      form.removeFieldValidator('field1', validator);
      form.clearErrors();
      (0, _chai.expect)(form.validate()).to.eql(true);
    });
  });
  describe('setLoad', function () {
    it('can load state from a promise', function (done) {
      var form = new _Form2.default('test').setLoad(function () {
        return Promise.resolve({
          field1: 'field1',
          field2: 'filed2'
        });
      });
      form.load().then(function () {
        done();
      });
    });
  });
  describe('setOnSuccess', function () {
    it('can provide onSuccess callback', function () {
      var onSuccess = function onSuccess() {
        return null;
      };
      var form = new _Form2.default('test').setOnSuccess(onSuccess);
      (0, _chai.expect)(form.onSuccess).to.eql(onSuccess);
    });
  });
  describe('setOnFailure', function () {
    it('can provide onFailure callback', function () {
      var onFailure = function onFailure() {
        return null;
      };
      var form = new _Form2.default('test').setOnFailure(onFailure);
      (0, _chai.expect)(form.onFailure).to.eql(onFailure);
    });
  });
  describe('getFieldErrors', function () {
    it('can get a map of fields and errors', function () {
      var form = new _Form2.default('test', {
        fields: {
          field1: {
            errors: ['error1']
          },
          field2: {
            errors: ['error2']
          },
          field3: {}
        }
      });

      (0, _chai.expect)(form.getFieldErrors()).to.eql((0, _immutable.Map)({
        field1: (0, _immutable.Stack)(['error1']),
        field2: (0, _immutable.Stack)(['error2']),
        field3: (0, _immutable.Stack)()
      }));
    });
  });
  describe('getFields', function () {
    it('can get a map of fields', function () {
      var form = new _Form2.default('test', {
        fields: {
          field1: {
            value: 'value1',
            errors: ['error1']
          }
        }
      });
      (0, _chai.expect)(form.getFields().toJS()).to.eql({
        field1: {
          value: 'value1',
          errors: ['error1']
        }
      });
    });
  });
  describe('reset', function () {
    it('can reset to the initial state', function () {
      var form = new _Form2.default('test', {
        fields: {
          field1: {
            value: ''
          }
        }
      });
      form.setField('field1', 'value1');
      form.setField('field2', 'value2');
      form.reset();
      (0, _chai.expect)(form.getState().toJS()).to.eql({
        fields: {
          field1: {
            value: '',
            errors: []
          }
        },
        errors: []
      });
    });
  });
  describe('saveInitialState', function () {
    it('can save a new initial state', function () {
      var form = new _Form2.default('test', {
        fields: {
          field1: {
            value: ''
          }
        }
      });
      form.setField('field1', 'value1');
      form.saveInitialState();
      form.setField('field2', 'value2');
      form.reset();
      (0, _chai.expect)(form.getState().toJS()).to.eql({
        fields: {
          field1: {
            value: 'value1',
            errors: []
          }
        },
        errors: []
      });
    });
  });
});